@@ -407,30 +407,48 @@ weightit2npcbps.cont <- function(formula, data, subset, estimand, ...) {
 }
 
 #Entropy balancing with ebal
-weightit2ebal <- function(formula, data, s.weights, subset, estimand, stabilize, ...) {
+weightit2ebal <- function(formula, data, s.weights, subset, estimand, stabilize, moments, int, ...) {
   A <- list(...)
 
   tt <- terms(formula)
   mf <- model.frame(tt, data[subset, , drop = FALSE], drop.unused.levels = TRUE)
   treat <- model.response(mf)
-  covs <- apply(model.matrix(tt, data=mf)[, -1, drop = FALSE], 2, make.closer.to.1)
+  covs <- model.matrix(tt, data=mf)[, -1, drop = FALSE]
+  covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+  covs <- apply(covs, 2, make.closer.to.1)
+
+  for (f in names(formals(ebal::ebalance))) {
+    if (length(A[[f]]) == 0) A[[f]] <- formals(ebal::ebalance)[[f]]
+  }
+  if (stabilize) {
+    for (f in names(formals(ebal::ebalance.trim))) {
+      if (length(A[[f]]) == 0) A[[f]] <- formals(ebal::ebalance.trim)[[f]]
+    }
+  }
 
   if (check.package("ebal")) {
     if (estimand %in% c("ATT", "ATC")) {
       if (estimand == "ATC" ) treat_ <- 1 - treat
       else treat_ <- treat
 
-      covs <- covs[, Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(covs[treat_ == j, , drop = FALSE])))), drop = FALSE]
+      covs <- covs[, colnames(covs) %in% Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs[treat_ == j, , drop = FALSE], treat_[treat_ == j])))))]
 
       covs[treat_ == 1,] <- covs[treat_ == 1,] * s.weights[subset][treat_ == 1] * sum(treat_ == 1)/ sum(s.weights[subset][treat_ == 1])
 
       ebal.out <- ebal::ebalance(Treatment = treat_, X = covs,
-                                 print.level = 3,
-                                 #base.weight = s.weights[subset][treat_ == 0],
-                                 ...)
-      if (stabilize) ebal.out <- ebal::ebalance.trim(ebal.out,
-                                                     print.level = 3,
-                                                     ...)
+                                 base.weight = A$base.weight,
+                                 norm.constant = A$norm.constant,
+                                 coefs = A$coefs,
+                                 max.iterations = A$max.iterations,
+                                 constraint.tolerance = A$constraint.tolerance,
+                                 print.level = 3)
+      if (stabilize) ebal.out <- ebal::ebalance.trim(ebalanceobj = ebal.out,
+                                                     max.weight = A$max.weight,
+                                                     min.weight = A$min.weight,
+                                                     max.trim.iterations = A$max.trim.iterations,
+                                                     max.weight.increment = A$max.weight.increment,
+                                                     min.weight.increment = A$min.weight.increment,
+                                                     print.level = 3)
 
       w <- cobalt::get.w(ebal.out, treat = treat_)
       w[treat_ == 0] <- w[treat_ == 0] / s.weights[subset][treat_ == 0]
@@ -444,14 +462,23 @@ weightit2ebal <- function(formula, data, s.weights, subset, estimand, stabilize,
         covs_i <- rbind(covs, covs[treat==i, , drop = FALSE])
         treat_i <- c(rep(1, nrow(covs)), rep(0, sum(treat==i)))
 
-        covs_i <- covs_i[, Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(covs_i[treat_i == j, , drop = FALSE]))))]
-
+        covs_i <- covs_i[, colnames(covs_i) %in% Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_i[treat_i == j, , drop = FALSE], treat_i[treat_i == j])))))]
         covs_i[treat_i == 1,] <- covs_i[treat_i == 1,] * s.weights[subset] * sum(treat_i == 1) / sum(s.weights[subset])
 
         ebal.out_i <- ebal::ebalance(Treatment = treat_i, X = covs_i,
-                                     #base.weight = s.weights[subset][treat==i],
-                                     print.level = 3, ...)
-        if (stabilize) ebal.out_i <- ebal::ebalance.trim(ebal.out_i, ...)
+                                     base.weight = A$base.weight,
+                                     norm.constant = A$norm.constant,
+                                     coefs = A$coefs,
+                                     max.iterations = A$max.iterations,
+                                     constraint.tolerance = A$constraint.tolerance,
+                                     print.level = 3)
+        if (stabilize) ebal.out_i <- ebal::ebalance.trim(ebalanceobj = ebal.out,
+                                                         max.weight = A$max.weight,
+                                                         min.weight = A$min.weight,
+                                                         max.trim.iterations = A$max.trim.iterations,
+                                                         max.weight.increment = A$max.weight.increment,
+                                                         min.weight.increment = A$min.weight.increment,
+                                                         print.level = 3)
 
         w[treat == i] <- ebal.out_i$w / s.weights[subset][treat == i]
       }
@@ -462,13 +489,24 @@ weightit2ebal <- function(formula, data, s.weights, subset, estimand, stabilize,
   return(obj)
 
 }
-weightit2ebal.multi <- function(formula, data, s.weights, subset, estimand, focal, stabilize, ...) {
+weightit2ebal.multi <- function(formula, data, s.weights, subset, estimand, focal, stabilize, moments, int, ...) {
   A <- list(...)
 
   tt <- terms(formula)
   mf <- model.frame(tt, data[subset,], drop.unused.levels = TRUE)
   treat <- model.response(mf)
-  covs <- apply(model.matrix(tt, data=mf)[,-1, drop = FALSE], 2, make.closer.to.1)
+  covs <- model.matrix(tt, data=mf)[, -1, drop = FALSE]
+  covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+  covs <- apply(covs, 2, make.closer.to.1)
+
+  for (f in names(formals(ebal::ebalance))) {
+    if (length(A[[f]]) == 0) A[[f]] <- formals(ebal::ebalance)[[f]]
+  }
+  if (stabilize) {
+    for (f in names(formals(ebal::ebalance.trim))) {
+      if (length(A[[f]]) == 0) A[[f]] <- formals(ebal::ebalance.trim)[[f]]
+    }
+  }
 
   if (check.package("ebal")) {
     if (estimand %in% c("ATT")) {
@@ -479,15 +517,24 @@ weightit2ebal.multi <- function(formula, data, s.weights, subset, estimand, foca
         treat_ <- ifelse(treat[treat %in% c(focal, i)] == i, 0, 1)
         covs_ <- covs[treat %in% c(focal, i), , drop = FALSE]
 
-        covs_ <- covs_[, Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(covs_[treat_ == j, , drop = FALSE]))))]
+        covs_ <- covs_[, colnames(covs_) %in% Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_[treat_ == j, , drop = FALSE], treat_[treat_ == j])))))]
 
         covs_[treat_ == 1,] <- covs_[treat_ == 1,] * s.weights[subset][treat == focal] * sum(treat == focal)/ sum(s.weights[subset][treat == focal])
 
         ebal.out <- ebal::ebalance(Treatment = treat_, X = covs_,
-                                   base.weight = s.weights[subset][treat == i],
-                                   print.level = 3, ...)
-        if (stabilize) ebal.out <- ebal::ebalance.trim(ebal.out,
-                                                       print.level = 3, ...)
+                                   base.weight = A$base.weight,
+                                   norm.constant = A$norm.constant,
+                                   coefs = A$coefs,
+                                   max.iterations = A$max.iterations,
+                                   constraint.tolerance = A$constraint.tolerance,
+                                   print.level = 3)
+        if (stabilize) ebal.out <- ebal::ebalance.trim(ebalanceobj = ebal.out,
+                                                       max.weight = A$max.weight,
+                                                       min.weight = A$min.weight,
+                                                       max.trim.iterations = A$max.trim.iterations,
+                                                       max.weight.increment = A$max.weight.increment,
+                                                       min.weight.increment = A$min.weight.increment,
+                                                       print.level = 3)
         w[treat == i] <- ebal.out$w / s.weights[subset][treat_ == i]
 
       }
@@ -499,17 +546,25 @@ weightit2ebal.multi <- function(formula, data, s.weights, subset, estimand, foca
         covs_i <- rbind(covs, covs[treat==i, , drop = FALSE])
         treat_i <- c(rep(1, nrow(covs)), rep(0, sum(treat==i)))
 
-        covs_i <- covs_i[, Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(covs_i[treat_i == j, , drop = FALSE]))))]
+        covs_i <- covs_i[, colnames(covs_i) %in% Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_i[treat_i == j, , drop = FALSE], treat_i[treat_i == j])))))]
 
         covs_i[treat_i == 1,] <- covs_i[treat_i == 1,] * s.weights[subset] * sum(treat_i == 1) / sum(s.weights[subset])
 
         ebal.out_i <- ebal::ebalance(Treatment = treat_i, X = covs_i,
-                                     base.weight = s.weights[subset][treat == i],
-                                     print.level = 3, ...)
+                                     base.weight = A$base.weight,
+                                     norm.constant = A$norm.constant,
+                                     coefs = A$coefs,
+                                     max.iterations = A$max.iterations,
+                                     constraint.tolerance = A$constraint.tolerance,
+                                     print.level = 3)
 
-        if (stabilize) ebal.out_i <- ebal::ebalance.trim(ebal.out_i,
-                                                         print.level = 3,
-                                                         ...)
+        if (stabilize) ebal.out_i <- ebal::ebalance.trim(ebalanceobj = ebal.out,
+                                                         max.weight = A$max.weight,
+                                                         min.weight = A$min.weight,
+                                                         max.trim.iterations = A$max.trim.iterations,
+                                                         max.weight.increment = A$max.weight.increment,
+                                                         min.weight.increment = A$min.weight.increment,
+                                                         print.level = 3)
 
         w[treat == i] <- ebal.out_i$w / s.weights[subset][treat == i]
       }
@@ -520,14 +575,15 @@ weightit2ebal.multi <- function(formula, data, s.weights, subset, estimand, foca
 }
 
 #Empirical Balancing Calibration weights with ATE
-weightit2ebcw <- function(formula, data, s.weights, subset, estimand, ...) {
+weightit2ebcw <- function(formula, data, s.weights, subset, estimand, moments, int, ...) {
   A <- list(...)
 
   tt <- terms(formula)
   mf <- model.frame(tt, data[subset,], drop.unused.levels = TRUE)
   treat <- model.response(mf)
-  covs <- apply(model.matrix(tt, data=mf)[,-1, drop = FALSE], 2, make.closer.to.1)
-
+  covs <- model.matrix(tt, data=mf)[, -1, drop = FALSE]
+  covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+  covs <- apply(covs, 2, make.closer.to.1)
 
   if (check.package("ATE")) {
     if (estimand %in% c("ATT", "ATC")) {
@@ -535,24 +591,17 @@ weightit2ebcw <- function(formula, data, s.weights, subset, estimand, ...) {
       else treat_ <- treat
       Y <- rep(0, length(treat_))
 
+      covs <- covs[, colnames(covs) %in% Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs[treat_ == j, , drop = FALSE], treat_[treat_ == j])))))]
+
       covs[treat_ == 1,] <- covs[treat_ == 1,] * s.weights[subset][treat_ == 1] * sum(treat_ == 1)/ sum(s.weights[subset][treat_ == 1])
 
       ate.out <- ATE::ATE(Y = Y, Ti = treat_, X = covs,
-                          ATT = TRUE, ...)
+                          ATT = TRUE,
+                          verbose = TRUE,
+                          ...)
       w <- ate.out$weights.q / s.weights[subset]
       w[treat_ == 1] <- 1
     }
-    # else if (estimand == "ATE") {
-    #   Y <- rep(0, length(treat))
-    #   for (i in unique(treat)) {
-    #     covs[treat == i,] <- covs[treat == i,] * s.weights[subset][treat == i] * sum(treat == i) / sum(s.weights[subset][treat == i])
-    #   }
-    #
-    #   ate.out <- ATE::ATE(Y = Y, Ti = treat, X = covs,
-    #                       ATT = FALSE, ...)
-    #
-    #   w <- (ate.out$weights.q + ate.out$weights.p)
-    # }
     else if (estimand == "ATE") {
       w <- rep(1, length(treat))
 
@@ -563,12 +612,13 @@ weightit2ebcw <- function(formula, data, s.weights, subset, estimand, ...) {
         treat_i <- c(rep(1, nrow(covs)), rep(0, sum(treat==i)))
         Y <- rep(0, length(treat_i))
 
-        covs_i <- covs_i[, Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(covs_i[treat_i == j, , drop = FALSE]))))]
+        covs_i <- covs_i[, colnames(covs_i) %in% Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_i[treat_i == j, , drop = FALSE], treat_i[treat_i == j])))))]
 
         covs_i[treat_i == 1,] <- covs_i[treat_i == 1,] * s.weights[subset] * sum(treat_i == 1) / sum(s.weights[subset])
 
         ate.out <- ATE::ATE(Y = Y, Ti = treat_i, X = covs_i,
-                            ATT = TRUE, ...)
+                            ATT = TRUE,
+                            verbose = TRUE, ...)
         w[treat == i] <- ate.out$weights.q[treat_i == 0] / s.weights[subset][treat == i]
 
       }
@@ -577,59 +627,60 @@ weightit2ebcw <- function(formula, data, s.weights, subset, estimand, ...) {
   obj <- list(w = w)
   return(obj)
 }
-weightit2ebcw.multi <- function(formula, data, s.weights, subset, estimand, focal, ...) {
+weightit2ebcw.multi <- function(formula, data, s.weights, subset, estimand, focal, moments, int, ...) {
   A <- list(...)
 
   tt <- terms(formula)
   mf <- model.frame(tt, data[subset,], drop.unused.levels = TRUE)
-  treat <- as.numeric(model.response(mf)) - 1
-  covs <- apply(model.matrix(tt, data=mf)[,-1, drop = FALSE], 2, make.closer.to.1)
-  Y <- rep(0, length(treat))
+  treat <- model.response(mf)
+  covs <- model.matrix(tt, data=mf)[, -1, drop = FALSE]
+  covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+  covs <- apply(covs, 2, make.closer.to.1)
 
   if (check.package("ATE")) {
+    if (estimand == "ATT") {
+      w <- rep(1, length(treat)) * s.weights[subset]
+      control.levels <- levels(treat)[levels(treat) != focal]
 
-    ate.out <- ATE::ATE(Y = Y, Ti = treat, X = covs,
-                        ATT = FALSE, ...)
+      for (i in control.levels) {
+        treat_ <- ifelse(treat[treat %in% c(focal, i)] == i, 0, 1)
+        covs_ <- covs[treat %in% c(focal, i), , drop = FALSE]
 
-    w <- apply(ate.out$weights.mat, 2, sum)
-  }
+        Y <- rep(0, length(treat_))
 
-  if (estimand %in% c("ATT")) {
-    w <- rep(1, length(treat)) * s.weights[subset]
-    control.levels <- levels(treat)[levels(treat) != focal]
+        #covs_ <- covs_[, Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(covs_[treat_ == j, , drop = FALSE]))))]
+        covs_ <- covs_[, colnames(covs_) %in% Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_[treat_ == j, , drop = FALSE], treat_[treat_ == j])))))]
 
-    for (i in control.levels) {
-      treat_ <- ifelse(treat[treat %in% c(focal, i)] == i, 0, 1)
-      covs_ <- covs[treat %in% c(focal, i), , drop = FALSE]
+        covs_[treat_ == 1,] <- covs_[treat_ == 1,] * s.weights[subset][treat == focal] * sum(treat == focal)/ sum(s.weights[subset][treat == focal])
 
-      covs_ <- covs_[, Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(covs_[treat_ == j, , drop = FALSE]))))]
-
-      covs_[treat_ == 1,] <- covs_[treat_ == 1,] * s.weights[subset][treat == focal] * sum(treat == focal)/ sum(s.weights[subset][treat == focal])
-
-      ate.out <- ATE::ATE(Y = Y, Ti = treat_i, X = covs_i,
-                          ATT = TRUE, ...)
-      w[treat == i] <- ate.out$weights.q[treat_i == 0] / s.weights[subset][treat == i]
+        ate.out <- ATE::ATE(Y = Y, Ti = treat_, X = covs_,
+                            ATT = TRUE, ...)
+        w[treat == i] <- ate.out$weights.q[treat_ == 0] / s.weights[subset][treat == i]
 
+      }
     }
-  }
-  else if (estimand == "ATE") {
-    w <- rep(1, length(treat))
+    else if (estimand == "ATE") {
+      w <- rep(1, length(treat))
 
-    for (i in levels(treat)) {
-      covs_i <- rbind(covs, covs[treat==i, , drop = FALSE])
-      treat_i <- c(rep(1, nrow(covs)), rep(0, sum(treat==i)))
+      for (i in levels(treat)) {
+        covs_i <- rbind(covs, covs[treat==i, , drop = FALSE])
+        treat_i <- c(rep(1, nrow(covs)), rep(0, sum(treat==i)))
 
-      covs_i <- covs_i[, Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(covs_i[treat_i == j, , drop = FALSE]))))]
+        Y <- rep(0, length(treat_i))
 
-      covs_i[treat_i == 1,] <- covs_i[treat_i == 1,] * s.weights[subset] * sum(treat_i == 1) / sum(s.weights[subset])
+        #covs_i <- covs_i[, Reduce("intersect", lapply(unique(treat_i, nmax = 2), function(j) colnames(remove.collinearity(covs_i[treat_i == j, , drop = FALSE]))))]
+        covs_i <- covs_i[, colnames(covs_i) %in% Reduce("intersect", lapply(unique(treat_, nmax = 2), function(j) colnames(remove.collinearity(cbind(covs_i[treat_ == j, , drop = FALSE], treat_[treat_ == j])))))]
 
-      ate.out <- ATE::ATE(Y = Y, Ti = treat_i, X = covs_i,
-                          ATT = TRUE, ...)
-      w[treat == i] <- ate.out$weights.q[treat_i == 0] / s.weights[subset][treat == i]
+        covs_i[treat_i == 1,] <- covs_i[treat_i == 1,] * s.weights[subset] * sum(treat_i == 1) / sum(s.weights[subset])
+
+        ate.out <- ATE::ATE(Y = Y, Ti = treat_i, X = covs_i,
+                            ATT = TRUE, ...)
+        w[treat == i] <- ate.out$weights.q[treat_i == 0] / s.weights[subset][treat == i]
+      }
     }
+    obj <- list(w = w)
+    return(obj)
   }
-  obj <- list(w = w)
-  return(obj)
 }
 
 #Stable balancing weights with sbw
@@ -640,11 +691,18 @@ weightit2sbw.multi <- function(...) {
   stop("Stable balancing weights are not currently supported. Please choose another method.", call. = FALSE)
 }
 
-# weightit2sbw <- function(formula, data, s.weights, subset, estimand, ...) {
+#SBW--------
+# weightit2sbw <- function(formula, data, s.weights, subset, estimand, moments, int, ...) {
 #   A <- list(...)
 #
-#   if (check.package("sbw") && check.package("slam")) {
-#     attachNamespace("slam")
+#   if (check.package("sbw")) {
+#     check.package("slam")
+#     if (!"package:slam" %in% search()) {
+#       need.to.detach.slam <- TRUE
+#       attachNamespace("slam")
+#     } else {
+#       need.to.detach.slam <- FALSE
+#     }
 #
 #     if (length(A$l_norm) == 0) A$l_norm <- "l_2"
 #     if (length(A$solver) == 0) A$solver <- "quadprog"
@@ -654,15 +712,23 @@ weightit2sbw.multi <- function(...) {
 #     if (length(A$gap_stop) == 0) A$gap_stop <- TRUE
 #     if (length(A$adaptive_rho) == 0) A$adaptive_rho <- TRUE
 #
-#     check.package(A$solver); attachNamespace(A$solver)
+#     check.package(A$solver)
+#     if (!paste0("package:", A$solver) %in% search()) {
+#       need.to.detach.solver <- TRUE
+#       attachNamespace(A$solver)
+#     } else {
+#       need.to.detach.solver <- FALSE
+#     }
 #
 #     tt <- terms(formula)
 #     mf <- model.frame(tt, data[subset,], drop.unused.levels = TRUE)
 #     treat <- model.response(mf)
-#     covs0 <- model.matrix(tt, data=mf,
+#     covs <- model.matrix(tt, data=mf,
 #                           contrasts.arg = lapply(mf[sapply(mf, is.factor)],
 #                                                  contrasts, contrasts=FALSE))[,-1, drop = FALSE]
-#     covs <- apply(covs0, 2, make.closer.to.1) * s.weights[subset]
+#     covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+#     covs <- apply(covs, 2, make.closer.to.1) * s.weights[subset]
+#
 #     new.data <- setNames(data.frame(treat, covs), as.character(seq_len(1+ncol(covs))))
 #
 #     if (length(A$bal_tols) == 0) bal_tols <- rep(.0001, ncol(covs))
@@ -752,15 +818,25 @@ weightit2sbw.multi <- function(...) {
 #       w[w < 0] <- 0
 #     }
 #   }
+#
+#   if (need.to.detach.slam) detach("package:slam", character.only = TRUE)
+#   if (need.to.detach.solver) detach(paste0("package:", A$solver), character.only = TRUE)
+#
 #   obj <- list(w = w)
 #   return(obj)
 #
 # }
-# weightit2sbw.multi <- function(formula, data, s.weights, subset, estimand, focal, ...) {
+# weightit2sbw.multi <- function(formula, data, s.weights, subset, estimand, focal, moments, int, ...) {
 #   A <- list(...)
 #
-#   if (check.package("sbw") && check.package("slam")) {
-#     attachNamespace("slam")
+#   if (check.package("sbw")) {
+#     check.package("slam")
+#     if (!"package:slam" %in% search()) {
+#       need.to.detach.slam <- TRUE
+#       attachNamespace("slam")
+#     } else {
+#       need.to.detach.slam <- FALSE
+#     }
 #
 #     if (length(A$l_norm) == 0) A$l_norm <- "l_2"
 #     if (length(A$solver) == 0) A$solver <- "quadprog"
@@ -770,15 +846,23 @@ weightit2sbw.multi <- function(...) {
 #     if (length(A$gap_stop) == 0) A$gap_stop <- TRUE
 #     if (length(A$adaptive_rho) == 0) A$adaptive_rho <- TRUE
 #
-#     check.package(A$solver); attachNamespace(A$solver)
+#     check.package(A$solver)
+#     if (!paste0("package:", A$solver) %in% search()) {
+#       need.to.detach.solver <- TRUE
+#       attachNamespace(A$solver)
+#     } else {
+#       need.to.detach.solver <- FALSE
+#     }
 #
 #     tt <- terms(formula)
 #     mf <- model.frame(tt, data[subset,], drop.unused.levels = TRUE)
 #     treat <- model.response(mf)
-#     covs0 <- model.matrix(tt, data=mf,
+#     covs <- model.matrix(tt, data=mf,
 #                           contrasts.arg = lapply(mf[sapply(mf, is.factor)],
 #                                                  contrasts, contrasts=FALSE))[,-1, drop = FALSE]
-#     covs <- apply(covs0, 2, make.closer.to.1) * s.weights[subset]
+#     covs <- cbind(covs, int.poly.f(covs, poly = moments, int = int))
+#     covs <- apply(covs, 2, make.closer.to.1) * s.weights[subset]
+#
 #     new.data <- setNames(data.frame(treat, covs), as.character(seq_len(1+ncol(covs))))
 #
 #     if (length(A$bal_tols) == 0) bal_tols <- .0001
@@ -856,6 +940,10 @@ weightit2sbw.multi <- function(...) {
 #       w[w < 0] <- 0
 #     }
 #   }
+#
+#   if (need.to.detach.slam) detach("package:slam", character.only = TRUE)
+#   if (need.to.detach.solver) detach(paste0("package:", A$solver), character.only = TRUE)
+#
 #   obj <- list(w = w)
 #   return(obj)
 #
